﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Threading;
using System.Windows.Forms;
using System.Reflection;
using System.IO;
using System.Xml.Linq;
using System.Data;

using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using Styx.CommonBot.Inventory;
using Styx.Plugins;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;

namespace TabardSwitcher 
{
    public class RepTabardSwitcher : HBPlugin
    {
        //Globals
        public static LocalPlayer me { get; set; }
        public static readonly string RootLocaltion = Path.Combine(Path.GetDirectoryName(Assembly.GetEntryAssembly().Location), Path.Combine("Plugins", "RepTabardSwitcher"));
        public static readonly string FileLocations = "Plugins/RepTabardSwitcher/";
        public static bool canLog = true;

        //Dataset to store XML settings
        protected static DataSet dsData = new DataSet("data");
        protected static DataTable dtReputation = new DataTable("reputation");
        protected static DataColumn dcName = new DataColumn("name");
        protected static DataColumn dcEnabled = new DataColumn("enabeld");
        protected static DataColumn dcReputationID = new DataColumn("reputationid");
        protected static DataColumn dcTabardID = new DataColumn("tabardid");
        protected static DataColumn dcTabardName = new DataColumn("tabardname");

        //Attribute overrides
        public override string Name { get { return "RepTabardSwitcher"; } }
        public override string Author { get { return "TimTonnyBOOM"; } }
        public override bool WantButton { get { return true; } }
        public override string ButtonText { get { return "Settings"; } }
        public override Version Version { get { return _version; } }
        private readonly Version _version = new Version(1, 0, 0, 0);

        //Property overrides
        public override void OnButtonPress()
        {
            SettingsForm.ShowDialog();
        }

        //Settings form init
        private Form CheckForm;
        public Form SettingsForm
        {
            get
            {
                if (CheckForm == null)
                    CheckForm = new Settings();
                return CheckForm;
            }
        }

        //Constructor
        private RepTabardSwitcher()
        {
            dsData.Tables.Add(dtReputation);
            dtReputation.Columns.Add(dcName);
            dtReputation.Columns.Add(dcEnabled);
            dtReputation.Columns.Add(dcReputationID);
            dtReputation.Columns.Add(dcTabardID);
            dtReputation.Columns.Add(dcTabardName);
            try
            {
                FillDataSet("Settings.xml");
            }
            catch(Exception Err)
            {
                //MessageBox.Show(Err.ToString());
                Styx.Common.Logging.Write("[Tabard Switcher]: Could not find Settings.XML");
            }
        }

        public static void FillDataSet(string path)
        {
            dsData.Clear();
            dsData.ReadXml(FileLocations + path);
        }

        //Prevent spam to Log.
        private readonly WaitTimer _itemCheckTimer = WaitTimer.TenSeconds;

        public override void Pulse()
        {
            if (!_itemCheckTimer.IsFinished)
            {
                return;
            }
           
            //Reset the timer
            _itemCheckTimer.Reset();

            //Execute Check method
            DoCheck();
        }

        private static void DoCheck()
        {
            //Check if the equiped Tabard reputation is exalted.
            if (CurrentTabardExalted() == true)
            {
                SwitchTabard();
            }
        }

        #region Equip Item
        /*
         * The following equip item methods are copyed from AutoEquip plugin written by Nesox
         * Thank you Nesox.
         */
        private static void EquipItem(int bagIndex, int bagSlot, int targetSlot)
        {
            Lua.DoString(
                "ClearCursor(); PickupContainerItem({0}, {1}); EquipCursorItem({2}); if StaticPopup1Button1 and StaticPopup1Button1:IsVisible() then StaticPopup1Button1:Click(); end;",
                bagIndex + 1, bagSlot + 1, targetSlot);
        }

        private static void EquipItemIntoSlot(WoWItem item, InventorySlot slot)
        {
            EquipItem(item.BagIndex, item.BagSlot, (int)slot);
        }

        private static void EquipItem(WoWItem item)
        {
            EquipItem(item.BagIndex, item.BagSlot, (int)item.ItemInfo.EquipSlot);
        }

        #endregion

        private static string GetRepurationLevel(uint id)
        {
            return Styx.StyxWoW.Me.GetReputationLevelWith(id).ToString();
        }

        private static bool CheckForExalted(string status)
        {
            if (status == "Exalted")
            {
                return true;
            }
            else 
            {
                return false;
            } 
        }

        private static IEnumerable<WoWItem> GetEquippableTabards()
        {
            var equippableItems = new List<WoWItem>();
            foreach (WoWItem item in Styx.StyxWoW.Me.BagItems)
            {
                ItemInfo info = item.ItemInfo;
                if (info.InventoryType == Styx.InventoryType.Tabard)
                {
                    //Logging.Write("[Tabard Switcher]: "+item);
                    equippableItems.Add(item);
                } 
            }
            return equippableItems;
        }

        private static WoWItem GetEquipedTabard()
        {
            WoWItem[] items = Styx.StyxWoW.Me.Inventory.Equipped.Items;
            return items[18]; 
        }

        private static bool CurrentTabardExalted()
        {
            bool back = false;
            //Read fill dataset with new data
            FillDataSet("Settings.xml");

            //Making sure we can also equip a tabart if no tabart is equiped
            WoWItem equippedTabard = GetEquipedTabard();
            if (equippedTabard == null)
            {
                back = true;
                return back;
            }
            DataRow[] row = dtReputation.Select("tabardid='" + equippedTabard.Entry.ToString() + "'"); 

            //Get the reputation status of current Tabard
            uint parsed = Convert.ToUInt32(row[0][2].ToString());
            string reputationStatus = GetRepurationLevel(parsed);
            if (reputationStatus == "Exalted" || reputationStatus == null || reputationStatus == "")
            {
                back = true;
            }
            return back; 
        }

        private static void SwitchTabard()
        { 
            //Read fill dataset with new data
            FillDataSet("Settings.xml");

            //Code for Switching Tabard and checking if reputation is already exalted  
            foreach (WoWItem Tabard in GetEquippableTabards())
            {
                if (Tabard == null)
                {
                    if (canLog == true)
                    {
                        Styx.Common.Logging.Write("[Tabard Switcher]: Equiping could not find any tabards to equip.");
                        canLog = false;
                    }
                    break;
                }

                //Make sure we can log the message again.
                canLog = true;
                DataRow[] row = dtReputation.Select("tabardid='" + Tabard.Entry.ToString() + "'");

                //Get the reputation status of current Tabard
                uint parsed = Convert.ToUInt32(row[0][2].ToString());
                string reputationStatus = GetRepurationLevel(parsed);

                if (reputationStatus == null || reputationStatus == "")
                {
                    Styx.Common.Logging.Write("[Tabard Switcher]: Could not find a reputation status on" + Tabard.Name.ToString()+". Are you sure you added all information of your current equiped tabart?");  
                }

                if (reputationStatus != "Exalted")
                {
                    //Equip the Tabard
                    EquipItem(Tabard);
                    Styx.Common.Logging.Write("[Tabard Switcher]: Equiping " + Tabard.Name.ToString());
                    return;
                }
            }
        }
    }
}
