﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using Styx;
using Styx.Common;
using Styx.CommonBot;
using Styx.Helpers;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using Styx.Plugins;

namespace TabardSwitcher 
{
    public partial class Settings : Form
    {
        public static readonly string FileLocations = "Plugins/RepTabardSwitcher/";

        protected static DataSet dsData = new DataSet("data");
        protected static DataTable dtReputation = new DataTable("reputation");
        protected static DataColumn dcName = new DataColumn("name");
        protected static DataColumn dcEnabled = new DataColumn("enabeld");
        protected static DataColumn dcReputationID = new DataColumn("reputationid");
        protected static DataColumn dcTabardID = new DataColumn("tabardid");
        protected static DataColumn dcTabardName = new DataColumn("tabardname");

        public Settings()
        {
            InitializeComponent();
            dsData.Tables.Add(dtReputation);
            dtReputation.Columns.Add(dcName);
            dtReputation.Columns.Add(dcEnabled);
            dtReputation.Columns.Add(dcReputationID);
            dtReputation.Columns.Add(dcTabardID);
            dtReputation.Columns.Add(dcTabardName);
        }

        private void Settings_Load(object sender, EventArgs e)
        {
            ckLB.Items.Clear();

            try
            {
                FillDataSet("Settings.xml");
                FillCheckListBox(ckLB);
            }
            catch (Exception Err)
            {
                //MessageBox.Show(Err.ToString());
                Styx.Common.Logging.Write("[Tabard Switcher]: Could not find Settings.XML");
            }
        }

        public static void FillDataSet(string path)
        {
            dsData.Clear();
            dsData.ReadXml(FileLocations + path);
        }

        public static void RefreshDataSet(CheckedListBox ckLB)
        {
            try
            {
                ckLB.Items.Clear();
                FillDataSet("Settings.xml");
                FillCheckListBox(ckLB);
            }
            catch (Exception Err)
            {
                //MessageBox.Show(Err.ToString());
                Styx.Common.Logging.Write("[Tabard Switcher]: Could not find Settings.XML");
            }
        }

        private static void FillCheckListBox(CheckedListBox ckLB)
        {
            ckLB.Items.Clear();
            foreach (DataRow drRep in dtReputation.Rows)
            {
                ckLB.Items.Add(drRep["name"]);
            }
        }

        private static void DeleteSelectedItem(CheckedListBox ckLB)
        {
            foreach (Object ItemChecked in ckLB.CheckedItems)
            {
                String item = ItemChecked.ToString();
                DataRow[] row = dtReputation.Select("name='" + item + "'");

                DialogResult dialogResult = MessageBox.Show("Delete: " + item + " ?", "Delete", MessageBoxButtons.YesNo, MessageBoxIcon.Warning);
                if (dialogResult == DialogResult.Yes)
                {
                    MessageBox.Show(item + " is deleted!", "Delete succes");
                    row[0].Delete();
                    dsData.WriteXml(FileLocations + "Settings.xml");
                }
                else if (dialogResult == DialogResult.No)
                {
                    MessageBox.Show(item + " is not deleted", "Delete cancelled");
                }
            }

            ckLB.Items.Clear();
            RefreshDataSet(ckLB);
        }

        private static void AddItem(CheckedListBox ckLB, TextBox name, TextBox RepID, TextBox TabardID, TextBox TabardName)
        {
            if (name.Text == "" || RepID.Text == "" || TabardID.Text == "" || TabardName.Text == "")
            {
                MessageBox.Show("Make sure you filled out everything", "Warning");
                return;
            }

            DataRow drRep = dtReputation.NewRow();
            drRep["name"] = name.Text.ToString();
            drRep["enabeld"] = "false";
            drRep["reputationid"] = RepID.Text.ToString();
            drRep["tabardid"] = TabardID.Text.ToString();
            drRep["tabardname"] = TabardName.Text.ToString();

            dtReputation.Rows.Add(drRep);
            dsData.WriteXml(FileLocations + "Settings.xml");

            ckLB.Items.Clear();
            RefreshDataSet(ckLB);    
        }

        private static void InsertItemsToTextBoxes(CheckedListBox ckLB, TextBox name, TextBox RepID, TextBox TabardID, TextBox TabardName)
        {
            //Select item from Checked list box
            String item = ckLB.SelectedItem.ToString();
            DataRow[] row = dtReputation.Select("name='" + item + "'");
  
            //Set tex boxes text
            name.Text = row[0][0].ToString();
            RepID.Text = row[0][2].ToString();
            TabardID.Text = row[0][3].ToString();
            TabardName.Text = row[0][4].ToString();
        }

        private static void SaveChangedItems(CheckedListBox ckLB, TextBox name, TextBox RepID, TextBox TabardID, TextBox TabardName)
        {
            if (name.Text == "" || RepID.Text == "" || TabardID.Text == "" || TabardName.Text == "")
            {
                MessageBox.Show("Make sure you filled out everything", "Warning");
                return;
            }

            //Select item by name
            String item = name.Text.ToString();
            DataRow[] row = dtReputation.Select("name='" + item + "'");

            //Change datatable values
            row[0][0] = name.Text.ToString();
            row[0][2] = RepID.Text.ToString();
            row[0][3] = TabardID.Text.ToString();
            row[0][4] = TabardName.Text.ToString();

            //Save data
            dsData.WriteXml(FileLocations + "Settings.xml");
           
            RefreshDataSet(ckLB);  
        }

        private void deleteBTN_Click(object sender, EventArgs e)
        {
            DeleteSelectedItem(ckLB);
        }

        private void addBTN_Click(object sender, EventArgs e)
        {
            AddItem(ckLB, nameTBX, reputationIdTBX, TabardIdTBX, TabardNameTBX);
        }

        private void ckLB_SelectedIndexChanged(object sender, EventArgs e)
        {
            InsertItemsToTextBoxes(ckLB, changeNameTBX, changeReputationIdTBX, changeTabardIdTBX, changeTabardNameTBX);          
        }

        private void Settings_Deactivate(object sender, EventArgs e)
        {
            //Form close message
        }

        private void changeBTN_Click(object sender, EventArgs e)
        {
            SaveChangedItems(ckLB, changeNameTBX, changeReputationIdTBX, changeTabardIdTBX, changeTabardNameTBX); 
        }

        private void refreshBTN_Click(object sender, EventArgs e)
        {
            RefreshDataSet(ckLB); 
        }
    }
}
